///////////////////////////////////////////////////////////////////////////////
//
//  Filename: BCommon.cpp
//  Author: Wang, Yi
//  Last modified: 2004-2-24 14:30
//
///////////////////////////////////////////////////////////////////////////////

#include "BCommon.h"

const double pi=3.1415926535897932384626433832795;
const double _2pi=2*pi;

tagPara m_Para;

// for debug
ofstream m_Log;

/*----------------------  return fp Value x mod a like integer value  --------*/
double _fmod(double x, double a)
{
	x /= a;
	x = (x-(int)x)*a;
	if (x<0) x += a;
	return x;
}

/*----------------------  get the Dist between the mittles of two circles  ---*/
double getDist(double x1, double y1, double x2, double y2)
{
	double dx=x1-x2;
	double dy=y1-y2;
	return sqrt(dx*dx+dy*dy);
}

/*----------------------  get two tangent circles of a vector  ---------------*/
void getVectTanCircle(tagVect &v, tagCircle &cl, tagCircle &cr, double r)
{
	double delta=pi/2;
	double tl=v.alpha-delta;
	double tr=v.alpha+delta;
	cl.r=r; cr.r=r;
	cl.ox=v.x+r*cos(tl);
	cl.oy=v.y+r*sin(tl);
	cr.ox=v.x+r*cos(tr);
	cr.oy=v.y+r*sin(tr);
	return;
}

/*----------------------  GetCrossVector of two Vectors  ---------------------*/
/*
 *  vc: output CrossVector
 *  return: false when pararell
 */
bool getCrossVect(tagVect &v1, tagVect &v2, tagVect &vc)
{
    double c1, c2, m;
    double sin1, sin2, cos1, cos2;
    
    sin1=sin(v1.alpha); cos1=cos(v1.alpha);
    sin2=sin(v2.alpha); cos2=cos(v2.alpha);
    m=sin2*cos1-sin1*cos2;
    if (m){
        c1=v1.x*sin1-v1.y*cos1;
        c2=v2.x*sin2-v2.y*cos2;
        vc.x=(c2*cos1-c1*cos2)/m;
        vc.y=(c2*sin1-c1*sin2)/m;
        vc.alpha=(v1.alpha+v2.alpha)/2;
        vc.alpha += (v2.alpha>v1.alpha)?pi/2:-pi/2;
        return true;
    }else{
		double m1=cos2*cos1-sin2*sin1;
		vc.alpha = (m1>0)?0:pi;
        return false;
    }
}

////////////////////////////////////////////////////////////////////////////////
// 
// for debug
//

/*----------------------------------------------------------------------------*/
/*
 * Description: dump data to debug.log (for debug)
 * Para<vect>:  a vect
 * Para<id>:  index of the vect
 * Return: none
 */
void _dump( tagVect &vect, int id )
{
	m_Log << "<Vector " << id << "> " << "; "
		<< "x: "  << vect.x << "; "
		<< "y: "  << vect.y << "; "
		<< "alpha: "  << vect.alpha << "; "
		<< "</Vector>"  << endl;
}

/*----------------------------------------------------------------------------*/
/*
 * Description: dump data to debug.log (for debug)
 * Para<circle>:  a circle
 * Para<id>:  index of the circle
 * Return: none
 */
void _dump(tagCircle &circle, int id)
{
	m_Log << "<Circle " << id << "> " << "; "
		<< "ox: "  << circle.ox << "; " 
		<< "oy: "  << circle.oy << "; " 
		<< "r: "  << circle.r << "; "
		<< "</Circle>"  << endl;
}

/*----------------------------------------------------------------------------*/
/*
 * Description: dump data to debug.log (for debug)
 * Para<arc>:  a arc
 * Para<id>:  index of the arc
 * Return: none
 */
void _dump(tagArc &arc, int id)
{
	m_Log << "<Arc " << id << "> "
		<< "a: "  << arc.a << "; " 
		<< "alen: "  << arc.alen << "; " <<  "c: ";
		_dump(arc.c, 0);
	m_Log << "</Arc>"  << endl;
}

/*----------------------------------------------------------------------------*/
/*
 * Description: dump data to text (for debug)
 * Para<cpt>:  a control point
 * Para<id>:  index of the control point
 * Return: none
 */
void _dump( tagCtlPoint &cpt, int id )
{

	m_Log << "<CtlPoint " << id << "> " << endl 
		<< "time: "  << cpt.time << "; " 
		<< "Type: "  << cpt.Type << "; " 
		<< "Value: "  << cpt.Value << "; " <<endl;
	m_Log << "</CtlPoint>" << endl; 
}

/*----------------------------------------------------------------------------*/
/*
 * Description: dump data to text (for debug)
 * Para<BDPt>:  a design point
 * Para<id>:  index of the design point
 * Return: none
 */
void _dump(tagBahnDesignPt &BDPt, int id)
{
	m_Log << "<BahnDesignPt " << id << "> " << endl 
		<< "time: "  << BDPt.time << endl 
		<< "Speed: "  << BDPt.Speed << endl 
		<< "StkIdSt: "  << BDPt.StkIdSt << endl 
		<< "StkCount: "  << BDPt.StkCount << endl << "vect: ";
		_dump(BDPt.Vect, 0);
	m_Log << "</BahnDesignPt>" << endl; 
}

/*----------------------------------------------------------------------------*/
/*
 * Description: dump data to text (for debug)
 * Para<cpt>:  a Strecke
 * Para<id>:  index of the Strecke
 * Return: none
 */
void _dump(tagBahnStrecke &stk, int id)
{
	m_Log << "<BahnStrecke " << id << "> " << endl 
		<< "Type: "  << stk.Type << "; " 
		<< "length: "  << stk.length << "; " 
		<< "TimeSt: "  << stk.TimeSt << "; " << endl << "VectSt: ";
		_dump(stk.VectSt, 0);
	m_Log << "VectEd: ";
		_dump(stk.VectEd, 0);
	m_Log << "arc: ";
		_dump(stk.arc, 0);
	m_Log << "</BahnStrecke>" << endl; 
}


/*----------------------------------------------------------------------------*/
/*
 * Description: dump data to text (for debug)
 * Para<cpt>:  a SpeedListItem
 * Para<id>:  index of the SpeedListItem
 * Return: none
 */
void _dump(tagSpeedListItem &spli, int id)
{
	m_Log << "<SpeedListItem " << id << "> " << endl 
		<< "BDPtId: " << spli.BDPtId << "; "
		<< "mode: " << spli.mode << "; "
		<< "t0: " << spli.t0 << "; " 
		<< "v0: " << spli.v0 << "; "
		<< "dt: " << spli.dt << "; "
		<< "a: "  << spli.a  << "; "
		<< "l: "  << spli.l << endl;
	m_Log << "</SpeedListItem>" << endl; 
}



