///////////////////////////////////////////////////////////////////////////////
//
//  Filename: CtlPointList.cpp
//  Author: Wang, Yi
//  Last modified: 2004-2-25 0:47
//
///////////////////////////////////////////////////////////////////////////////

#include "CtlPointList.h"
	
/*----------------------------------------------------------------------------*/
/*
 * Description: reset the list
 * Para: none
 * Return: none
 */
void CtlPointList::Reset(tagBahnStrecke Stks[], int StkCt, SpeedList &SpList)
{
	list.setAutoDelete(true);
	list.clear();

	Append(SpList);
	Append(Stks, StkCt);
	list.sort();
}

/*----------------------------------------------------------------------------*/
/*
 * Description: append a speedlist to the list
 * Para<SpList>: the speedlist
 * Return: none
 */
void CtlPointList::Append(SpeedList &SpdList)
{
	int i, Ct;
	
	Ct = SpdList.GetCount();

	for (i=0;i<Ct;i++)
	{
		tagSpeedListItem &SpdItem = SpdList.Item(i);
		
		if (SpdItem.a != 0)
		{
			tagCtlPoint *pItem = new tagCtlPoint;
			pItem->time = SpdItem.t0;
			pItem->Type = 1;
			pItem->Value = SpdList.Item(i+1).v0;
			list.append(pItem);
		}
	}
}

/*----------------------------------------------------------------------------*/
/*
 * Description: append a Streckelist to the list
 * Para<Stks>: the Streckelist
 * Para<StkCt>: length of the Streckelist
 * Return: none
 */
void CtlPointList::Append(tagBahnStrecke Stks[], int StkCt)
{
	for (int i=0;i<StkCt;i++)
	{
		tagCtlPoint *pItem = new tagCtlPoint;
		pItem->time = Stks[i].TimeSt;
		pItem->Type = 0;

		if (Stks[i].Type==0)
		{
			pItem->Value =  0;
		}
		else
		{
			if (Stks[i].arc.alen>0)
			{
				pItem->Value = Stks[i].arc.c.r;
			}
			else
			{
				pItem->Value = -Stks[i].arc.c.r;
			}
		}
		list.append(pItem);
	}
}	

/*----------------------------------------------------------------------------*/
/*
 * Description: length of the list
 * Para: none
 * Return: count of listitems
 */
int CtlPointList::GetCount(void)
{
	return list.count();
}


/*----------------------------------------------------------------------------*/
/*
 * Description: length of the list
 * Para: none
 * Return: count of listitems
 */
tagCtlPoint &CtlPointList::Item(int index)
{
	return *(list.at(index));
}

/*----------------------------------------------------------------------------*/
/*
 * Description: length of the list
 * Para: none
 * Return: count of listitems
 */
QString CtlPointList::GetCmdF(int index)
{
	tagCtlPoint &cp = *(list.at(index));
	QString cmd;
	char c;
	unsigned short time = cp.time;
	unsigned short value;
	
	if (cp.Type==0)
	{
		c = 'L';
		value = cp.Value * m_Para.LenkFact + 0x2d00;
	}
	else
	{
		c = 'V';
		value = cp.Value * m_Para.SpeedFact;
	}

	return cmd.sprintf("$%c%04x%04x",c,time,value);
}

/*----------------------------------------------------------------------------*/
CtlPointList::~CtlPointList(void)
{
	list.clear();
}

/*----------------------------------------------------------------------------*/

int QGList::compareItems( QPtrCollection::Item s1, QPtrCollection::Item s2 )
{
	return ((tagCtlPoint *)s1)->time - ((tagCtlPoint *)s2)->time;
}
