///////////////////////////////////////////////////////////////////////////////
//
//  Filename: SpeedList.cpp
//  Author: Wang, Yi
//  Last modified: 2004-2-24 16:36
//
///////////////////////////////////////////////////////////////////////////////

#include "SpeedList.h"

/*----------------------------------------------------------------------------*/
/*
 * Description: add an item into the tail of the list
 * Para<pBDPt>: point to a BDPt
 * Para<pStk>: point to the first Strecke of the BDPt
 * Return: none
 */
void SpeedList::Append(tagBahnDesignPt *pBDPt, tagBahnStrecke *pStk, int tBDPtId)
{
	int i;
	tagSpeedListItem item;
	item.BDPtId = tBDPtId;

	// to prepare the context
	spc.IsErr = false;
	spc.tSt = pBDPt[0].time;
	spc.au = m_Para.PlusA;
	spc.ad = m_Para.NegA;
	spc.dTime = pBDPt[1].time - pBDPt[0].time;
	spc.vSt = pBDPt[0].Speed;
	spc.vEd = pBDPt[1].Speed;
	spc.dtSt = (spc.vSt==0) ? 0 : m_Para.MinSpeedHold / spc.vSt;
	spc.dtEd = (spc.vEd==0) ? 0 : m_Para.MinSpeedHold / spc.vEd;
	spc.tLen = 0;

	for (i=0; i<pBDPt->StkCount; i++)
	{
		spc.tLen += pStk[i].length;
	}

	// check the minial needed time
	double MinAccTime = (spc.vEd>=spc.vSt) ? (spc.vEd - spc.vSt) / spc.au 
									 : (spc.vSt - spc.vEd) / spc.ad;

	if (spc.dTime<(MinAccTime+spc.dtSt+spc.dtEd))
	{
		spc.IsErr = true;
		return;
	}

	// sMax, sMin, vMax, vMin
	double s0, s1, s2, s3;
	double t0, t1, t2, t3;
	double t = spc.dTime - spc.dtSt - spc.dtEd;
	double sMin0, vMin0;

	spc.tu = (spc.vEd - spc.vSt + spc.ad * t) / (spc.au + spc.ad);
	spc.td = t - spc.tu;

	spc.vMax = spc.vSt + spc.au * spc.tu;

	s0 = spc.dtSt * spc.vSt;
	s1 = 0.5 * spc.tu * (spc.vMax + spc.vSt);
	s2 = 0.5 * spc.td * (spc.vMax + spc.vEd);
	s3 = spc.dtEd * spc.vEd;
	spc.sMax = s0 + s1 + s2 + s3;
	
	vMin0 = spc.vSt - spc.ad * spc.td;
	s1 = 0.5 * spc.td * (vMin0 + spc.vSt);
	s2 = 0.5 * spc.tu * (vMin0 + spc.vEd);
	sMin0 = s0 + s1 + s2 + s3;

	if (vMin0>=0)
	{
		spc.vMin = vMin0;
		spc.sMin = sMin0;
	}
	else
	{
		spc.vMin = 0;
		t1 = spc.vSt / spc.ad;
		t2 = spc.vEd / spc.au;
		spc.sMin = 0.5 * ( t1 * spc.vSt + t2 * spc.vEd ) + s0 + s3;
	}

	if ( (spc.tLen>spc.sMax) || (spc.tLen<spc.sMin) )
	{
		spc.IsErr = true;
		return;
	}


	// mode
	double sMaxMid, sMinMid, v2, ds;

	if (spc.vEd>=spc.vSt)
	{
		t1 = MinAccTime;
		t2 = t - MinAccTime;
		s1 = 0.5 * t1 * (spc.vEd - spc.vSt);
		sMinMid = s0 + s1 + s3 + t2 * spc.vSt;
		sMaxMid = s0 + s1 + s3 + t2 * spc.vEd;

		if ((spc.tLen>=sMinMid)&&(spc.tLen<=sMaxMid))
		{ // mode 1
			spc.mode = 1;
			item.mode = 1;

			s2 = spc.tLen - sMinMid;
			t2 = s2 / spc.vEd;
			t0 = t - t2 - t1;
			s0 = t0 * spc.vSt;
			s2 += spc.dtEd * spc.vEd;
			t2 += spc.dtEd;
			s0 += spc.dtSt * spc.vSt;
			t0 += spc.dtSt;
		}
		else
		{ // mode 3
			spc.mode = 3;
			item.mode = 3;
			
			ds = spc.sMax - spc.tLen;
			t2 = sqrt( 2 * ds * (spc.au + spc.ad ) / (spc.au * spc.ad ) );
			v2 = spc.vMax - t2 * (spc.au * spc.ad ) / (spc.au + spc.ad );
			s2 = v2 * t2;

			if ( (v2>m_Para.MaxSpeed) || (t2>t) )
			{
				spc.IsErr = true;
				return;
			}
			
			t1 = (v2 - spc.vSt) / spc.au;
			s1 = spc.vSt * t1 + 0.5 * spc.au * t1 * t1;
			
			t3 = (v2 - spc.vEd) / spc.ad;
			s3 = spc.vEd * t3 + 0.5 * spc.ad * t3 * t3;
		}
	}
	else
	{
		t1 = MinAccTime;
		t2 = t - MinAccTime;
		s1 = 0.5 * MinAccTime * (spc.vSt - spc.vEd);
		sMinMid = s0 + s1 + s3 + t2 * spc.vEd;
		sMaxMid = s0 + s1 + s3 + t2 * spc.vSt;

		if ((spc.tLen>=sMinMid)&&(spc.tLen<=sMaxMid))
		{ // mode 2
			spc.mode = 2;
			item.mode = 2;

			s0 = spc.tLen - sMinMid;
			t0 = s0 / spc.vSt;
			t2 = t - t0 - t1;
			s2 = t2 * spc.vEd;

			s0 += spc.dtSt * spc.vSt;
			t0 += spc.dtSt;		
			s2 += spc.dtEd * spc.vEd;
			t2 += spc.dtEd;
		}
		else
		{
			spc.mode = 4;
			item.mode = 4;
			
			ds = spc.tLen - sMin0;
			t2 = sqrt( 2 * ds * (spc.au + spc.ad ) / (spc.au * spc.ad ) );
			v2 = vMin0 + t2 * (spc.au * spc.ad ) / (spc.au + spc.ad );
			s2 = v2 * t2;

			if (t2>t)
			{
				spc.IsErr = true;
				return;
			}
			
			t1 = ( spc.vSt - v2 ) / spc.ad;
			s1 = v2 * t1 + 0.5 * spc.ad * t1 * t1;
			
			t3 = ( spc.vEd - v2 ) / spc.au;
			s3 = v2 * t3 + 0.5 * spc.au * t3 * t3;
		}
	}

	switch (spc.mode)
	{
	case 1:
	case 2:
		{
			//
			pBDPt->SpdIdSt = vec.count();
			pBDPt->SpdCount = 3;

			// t0
			item.a = 0;
			item.dt = t0;
			item.v0 = spc.vSt;
			item.t0 = spc.tSt;
			item.l = s0;
			vec.append(item);
			
			// t1
			item.a = (spc.mode==1) ? spc.au : -spc.ad;
			item.dt = t1;
			item.v0 = spc.vSt;
			item.t0 += t0;
			item.l = s1;
			vec.append(item);
			
			// t2
			item.a = 0;
			item.dt = t2;
			item.v0 = spc.vEd;
			item.t0 += t1;
			item.l = s2;
			vec.append(item);
			
			break;
		}
	case 3:
	case 4:
		{
			//
			pBDPt->SpdIdSt = vec.count();
			pBDPt->SpdCount = 3;

			// t0
			item.a = 0;
			item.dt = spc.dtSt;
			item.v0 = spc.vSt;
			item.t0 = spc.tSt;
			item.l = spc.dtSt * spc.vSt;

			if (spc.dtSt > 0)
			{
				vec.append(item);
				pBDPt->SpdCount++;
			}
			
			// t1
			item.a = (spc.mode==3) ? spc.au : -spc.ad;
			item.dt = t1;
			item.v0 = spc.vSt;
			item.t0 += spc.dtSt;
			item.l = s1;
			vec.append(item);
			
			// t2
			item.a = 0;
			item.dt = t2;
			item.v0 = v2;
			item.t0 += t1;
			item.l = s2;
			vec.append(item);

			// t3
			item.a = (spc.mode==3) ? -spc.ad : spc.au;
			item.dt = t3;
			item.v0 = v2;
			item.t0 += t2;
			item.l = s3;
			vec.append(item);

			// t4
			item.a = 0;
			item.dt = spc.dtEd;
			item.v0 = spc.vEd;
			item.t0 += t3;
			item.l = spc.dtEd * spc.vEd;

			if (spc.dtEd > 0)
			{
				vec.append(item);
				pBDPt->SpdCount++;
			}
		}
	}

	FillTime(pBDPt, pStk);
}
	
/*----------------------------------------------------------------------------*/
/*
 * Description: access to an item
 * Para<index>: index of the item
 * Return: reference of the item
 */
tagSpeedListItem &SpeedList::Item(int index)
{
	return vec[index];
}

/*----------------------------------------------------------------------------*/
/*
 * Description: reset the list
 * Para: none
 * Return: none
 */
void SpeedList::Reset(void)
{
	vec.clear();
}

/*----------------------------------------------------------------------------*/
/*
 * Description: length of the list
 * Para: none
 * Return: count of listitems
 */
int SpeedList::GetCount(void)
{
	return vec.count();
}

/*----------------------------------------------------------------------------*/
/*
 * Description: Fill the time field of BahnStreckes of a DesignPt
 * Para<pBDPt>: point to the DesignPt
 * Para<pStk>: array of the BahnStreckes
 * Return: count of listitems
 */
void SpeedList::FillTime(tagBahnDesignPt *pBDPt, tagBahnStrecke *pStk)
{
	double tLength = pStk[0].length;
	pStk[0].TimeSt = pBDPt->time;	

	for (int i=1;i<pBDPt->StkCount;i++)
	{
		pStk[i].TimeSt = LengthToTime(pBDPt->SpdIdSt, tLength);
		tLength += pStk[i].length;
	}
}

/*----------------------------------------------------------------------------*/
/*
 * Description: get time from length
 * Para<IdSt>: first Id in SpeedList
 * Para<tLength>: the length
 * Return: time, -1 by error
 */
double SpeedList::LengthToTime(int IdSt, double tLength)
{
	double l = 0, time;
	int Ct = vec.count();
	
	for (int i=IdSt;i<Ct;i++)
	{
		tagSpeedListItem &tItem = vec[i];

		if (l+tItem.l>=tLength)
		{
			time = tItem.t0;

			if (tItem.a == 0)
			{
				time += (tLength-l) / tItem.v0; 
			}
			else
			{
				time += (sqrt( pow(tItem.v0, 2.0) / pow(tItem.a, 2.0) + 2 * 
						(tLength-l) / tItem.a) - ( tItem.v0 / tItem.a )); 
			}

			return time;
		}

		l += tItem.l;
	}

	return -1;
}